@@ -27,21 +27,7 @@ function rc_run() {
         exit 1
     fi
 
-    if rc_buildah; then
-        buildah bud --pull-always --tag "jss_$image:latest" -f "tools/Dockerfiles/$image" .
-        ret="$?"
-        if [ "x$ret" != "x0" ]; then
-            echo "Container build exited with status: $ret"
-            return $ret
-        fi
-
-        podman run "jss_$image:latest"
-        ret="$?"
-        if [ "x$ret" != "x0" ]; then
-            echo "Container run exited with status: $ret"
-            return $ret
-        fi
-    elif rc_docker; then
+    if rc_docker; then
         docker build --pull --tag "jss_$image:latest" -f "tools/Dockerfiles/$image" .
         ret="$?"
         if [ "x$ret" != "x0" ]; then
@@ -55,6 +41,20 @@ function rc_run() {
             echo "Container run exited with status: $ret"
             return $ret
         fi
+    elif rc_buildah; then
+        buildah bud --pull-always --tag "jss_$image:latest" -f "tools/Dockerfiles/$image" .
+        ret="$?"
+        if [ "x$ret" != "x0" ]; then
+            echo "Container build exited with status: $ret"
+            return $ret
+        fi
+
+        podman run "jss_$image:latest"
+        ret="$?"
+        if [ "x$ret" != "x0" ]; then
+            echo "Container run exited with status: $ret"
+            return $ret
+        fi
     else
         echo "No supported container platform; please rerun with podman" 1>&2
         echo "and buildah or docker installed." 1>&2
